//************************************************************************
//*                           BipNUM Win V0.9.1                          *
//*----------------------------------------------------------------------*
//*                   BipNUM Class for Microsoft Windows                 *
//*                                                                      *
//*                                                         P. Basset    *
//************************************************************************


#include <string>
#include <sstream>
#include "BipNUM.h"


// error codes for BP_Currency::iBPError ( return by BP_Currency::geterror() )
//
//  0 = no error
//  1 = malformed currency string in set command
//  2 = unknown standard format in setstdformat command
//  3 = format too small to display integer part
//  4 = integer format is bad
//  5 = decimal format is bad




//-----------------------------------------------------------------
//!    BP_Currency::BP_Currency : class constructor               !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::BP_Currency ()
{

	// reset currency and error code
	BP_Currency::reset ();

}



//-----------------------------------------------------------------
//!    BP_Currency::~BP_Currency : class destructor               !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::~BP_Currency ()
{
}



//-----------------------------------------------------------------
//!    BP_Currency::reset : reset currency and format             !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::reset (void)
{
	// reset currency value
	BP_Currency::setnull ();

	// default format is "US"
	BP_Currency::setstdformat (0, 0, "US");

	// reset error code
	iBPError = 0;
}



//-----------------------------------------------------------------
//!    BP_Currency::copyvalue : copy value of BP_Currency         !
//-----------------------------------------------------------------
//! BP_CurSource : source BP_Currency                             !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
//! rem : copy only the value of the currency,                    !
//!       don't copy the format                                   !
//-----------------------------------------------------------------
BP_Currency::copyvalue (BP_Currency BP_CurSource)
{
	// reset error code
	iBPError = 0;

	// copy value
	BP_Cur = BP_CurSource.BP_Cur;
}



//-----------------------------------------------------------------
//!    BP_Currency::copyformat : copy format of BP_Currency       !
//-----------------------------------------------------------------
//! BP_CurSource : source BP_Currency                             !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
//! rem : copy only the format of the currency,                   !
//!       don't copy the value                                    !
//-----------------------------------------------------------------
BP_Currency::copyformat (BP_Currency BP_CurSource)
{
	// reset error code
	iBPError = 0;

	// copy format
	BP_Fmt = BP_CurSource.BP_Fmt;
}



//-----------------------------------------------------------------
//!    BP_Currency::set : fill string currency into a BP_Currency !
//!                           container                           !
//-----------------------------------------------------------------
//! BP_SetString : currency value                                 !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::set (const std::string BP_SetString)
{

	int iTemp;
	std::string sInput;
	std::string sString;

	
	// reset error code
	iBPError = 0;

	// transfert input string
	sInput = BP_SetString;

	// if input string is not null
	if (sInput != "" && sInput != BP_Fmt.sBP_Null)
	{

		// delete currency symbol
		if (BP_Fmt.sBP_CurrencySymbol != "")
		{
			iTemp = sInput.find (BP_Fmt.sBP_CurrencySymbol);
			if (iTemp >= 0)
					sInput.erase(iTemp, BP_Fmt.sBP_CurrencySymbol.length());
		}

		// delete all leading char
		if (BP_Fmt.sBP_LeadChar != "")
		{
			iTemp = sInput.find (BP_Fmt.sBP_LeadChar);
			while (iTemp >= 0)
			{
				sInput.erase(iTemp, 1);
				iTemp = sInput.find (BP_Fmt.sBP_LeadChar);
			}
		}
		
		// delete all kilo sign
		if (BP_Fmt.sBP_KiloSign != "")
		{
			iTemp = sInput.find (BP_Fmt.sBP_KiloSign);
			while (iTemp >= 0)
			{
				sInput.erase(iTemp, BP_Fmt.sBP_KiloSign.length());
				iTemp = sInput.find (BP_Fmt.sBP_KiloSign);
			}
		}

		// delete leading spaces
		if (sInput [0] == ' ')
		{
			iTemp = sInput.find_first_not_of (" ");
			sInput = sInput.substr (iTemp);
		}
		
		// delete ending spaces
		iTemp = sInput.length() - 1;
		if (sInput [iTemp] == ' ')
		{
			while (iTemp >= 0 && sInput [iTemp] == ' ')
			{
				sInput.erase(iTemp, 1);
				iTemp--;
			}
		}

		// look for negative sign : "-"
		BP_Cur.bBP_Negative = false;
		iTemp = sInput.find ("-");
		if (iTemp >= 0)
		{
			BP_Cur.bBP_Negative = true;

			// erase negative sign
			sInput.erase(iTemp, 1);
		}

		// delete positive sign : "+"
		iTemp = sInput.find ("+");
		if (iTemp >= 0)
			sInput.erase(iTemp, 1);



		// verify if input string is numeric (the format was deleted before)
		sString = sInput;
		if (BP_Fmt.sBP_DecimalSign != "")
		{
			iTemp = sString.find (BP_Fmt.sBP_DecimalSign);
			if (iTemp >= 0)
				// delete temporary decimal sign
				sString.erase(iTemp, BP_Fmt.sBP_DecimalSign.length());
		}
		else
		{
			// suppose decimal sign is "."
			iTemp = sString.find (".");
			if (iTemp >= 0)
				// delete temporary decimal sign
				sString.erase(iTemp, 1);
			else
			// not found, so try with ","
			{
				iTemp = sString.find (",");
				if (iTemp >= 0)
					// delete temporary decimal sign
					sString.erase(iTemp, 1);
			}
		}

		iTemp = 0;
		while (iTemp < sString.length() && iBPError == 0)
		{
			if (sString [iTemp] < '0' || sString [iTemp] > '9')
				// input string is not numeric
				iBPError = 1;

			iTemp++;
		}



		// store currency value
		if (iBPError == 0)
		{
			// look for decimal sign
			iTemp = sInput.find (BP_Fmt.sBP_DecimalSign);

			// no decimal part
			if (iTemp < 1)
			{
				BP_Cur.lBP_Integer = _atoi64 (sInput.c_str());
				BP_Cur.bBP_Null = false;
			}
			else
			{
				// store integer part
				sString = sInput.substr (0, iTemp);
				BP_Cur.lBP_Integer = _atoi64 (sString.c_str());
				
				// store decimal part
				sString = "1" + sInput.substr (iTemp + BP_Fmt.sBP_DecimalSign.length());
				BP_Cur.lBP_Decimal = _atoi64 (sString.c_str());

				// currency is not null
				BP_Cur.bBP_Null = false;
			}

		}
		else
		{
			// error : set currency to null
			BP_Currency::setnull ();
		}
	}
	else
	{
		// currency is null
		BP_Currency::setnull ();
	}

}



//-----------------------------------------------------------------
//!    BP_Currency::setnull : set currency to null value          !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::setnull (void)
{
	// reset error code - no keep error code
//	iBPError = 0;

	// set currency to null
	BP_Cur.lBP_Integer = 0;
	BP_Cur.lBP_Decimal = 0;
	BP_Cur.bBP_Null = true;
	BP_Cur.bBP_Negative = false;
}



//-----------------------------------------------------------------
//!    BP_Currency::geterror : return error code                  !
//-----------------------------------------------------------------
//! return : error code or 0 if none                              !
//-----------------------------------------------------------------
int BP_Currency::geterror (void)
{

	return BP_Currency::iBPError;

}



//-----------------------------------------------------------------
//!    BP_Currency::str : return currency into a string           !
//!                        depending on format                    !
//-----------------------------------------------------------------
//! return : string                                               !
//-----------------------------------------------------------------
//! use BP_Currency::setformat before BP_Currency::str            !
//! if error occurs, the return string = "" and                   !
//!     BP_Currency:geterror() return error code                  !
//-----------------------------------------------------------------
std::string BP_Currency::str (void)
{
	std::ostringstream os;
	char c255 [255];
	std::string sString;
	int iTemp;


	// reset error code
	iBPError = 0;

	if (BP_Cur.bBP_Null != true)
	{
		// reset working string
		sString = "";

		// add 50 leading zero
		if (BP_Fmt.sBP_LeadChar != "")
			sString.append (50, BP_Fmt.sBP_LeadChar [0]);

		// display negative sign : "-"
		iTemp = 0;
		if (BP_Cur.bBP_Negative == true)
		{
			sString += "-";
			iTemp = 1;
		}
		
		// convert long long to string
		_i64toa (BP_Cur.lBP_Integer, c255, 10);
		sString += c255;

		// limit to format length
		iTemp += BP_Fmt.iBP_NumInt;
		if (iTemp < sString.length())
		{
			sString = sString.substr (sString.length() - iTemp, iTemp);
			iBPError = 3;
		}
		
		// insert kilo sign
		if (BP_Fmt.sBP_KiloSign != "")
		{
			iTemp = sString.length() - 3;
			while (iTemp > 0)
			{
				//
				if (sString [iTemp] != ' ' && sString [iTemp] != BP_Fmt.sBP_LeadChar [0])
					sString.insert(iTemp, BP_Fmt.sBP_KiloSign);

				iTemp -= 3;
			}
		}

		// display integer part
		os << sString;


		// if decimal part exists
		if (BP_Fmt.iBP_NumDec != 0)
		{
			//convert decimal part to string
			itoa (BP_Cur.lBP_Decimal, c255, 10);
			sString = c255;

			// display decimal sign
			if (BP_Fmt.iBP_NumDec > 0)
				os << BP_Fmt.sBP_DecimalSign;

			// add trailing zero and truncate decimal part
			sString += "000000000000";
			sString = sString.substr (0, BP_Fmt.iBP_NumDec + 1);
			
			// display decimal part, exclude first char, = "1"
			os << sString.substr (1);
		}

		// display currency symbol
		if (BP_Fmt.sBP_CurrencySymbol != "")
			os << " " << BP_Fmt.sBP_CurrencySymbol;

		return os.str();

	}
	else
		// currency is null;
		return BP_Fmt.sBP_Null;

}



//-----------------------------------------------------------------
//!    BP_Currency::isnull : return true if currency is null      !
//!                           else return false                   !
//-----------------------------------------------------------------
//! return : bool                                                 !
//-----------------------------------------------------------------
bool BP_Currency::isnull (void)
{
	// reset error code - no keep error code
//	iBPError = 0;

	return BP_Cur.bBP_Null;

}



//-----------------------------------------------------------------
//!    BP_Currency::setformat : set currency format               !
//!                         use by BP_Currency::str               !
//-----------------------------------------------------------------
//! iBP_SetNumInt : character to display before decimal sign      !
//! iBP_SetNumDec : character to display after decimal sign       !
//! sBP_SetDecimalSign : decimal sign (default ".")               !
//! sBP_SetKiloSign : kilo sign (ie : " " , "." or "")            !
//! sBP_SetCurrencySymbol : currency symbol (ie : "euro" or "$")  !
//! sBP_SetLeadChar : leading char display before number          !
//!                      (truncated to 1 char)                    !
//! sBP_SetNull : null format                                     !
//!                                                               !
//! samples :                                                     !
//! setformat(4,2,",",".","$","","") give "1.234,56 $"            !
//! setformat(3,0,"","","","0","") give "001"                     !
//! setformat(1,0,"","","","","NULL") give "NULL"                 !
//! setformat(10,2,",","","$","*","") give "******1234,56 $"      !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Currency::setformat (const int iBP_SetNumInt,
						const int iBP_SetNumDec,
						const std::string sBP_SetDecimalSign,
						const std::string sBP_SetKiloSign,
						const std::string sBP_SetCurrencySymbol,
						const std::string sBP_SetLeadChar,
						const std::string sBP_SetNull)
{
	// reset error code
	iBPError = 0;

	if (iBP_SetNumInt < 0 || iBP_SetNumInt > 18)
		iBPError = 4;

	if (iBP_SetNumDec < 0 || iBP_SetNumDec > 9)
		iBPError = 5;

	// store values
	BP_Fmt.iBP_NumInt = iBP_SetNumInt;
	BP_Fmt.iBP_NumDec = iBP_SetNumDec;
	BP_Fmt.sBP_DecimalSign = sBP_SetDecimalSign;
	BP_Fmt.sBP_KiloSign = sBP_SetKiloSign;
	BP_Fmt.sBP_CurrencySymbol = sBP_SetCurrencySymbol;
	BP_Fmt.sBP_LeadChar = sBP_SetLeadChar.substr (0, 1);
	BP_Fmt.sBP_Null = sBP_SetNull;

}



//-----------------------------------------------------------------
//!    BP_Currency::setformat : set currency format               !
//!                         for simple format                     !
//-----------------------------------------------------------------
//! iBP_SetFormat : format of the currency                        !
//!                                                               !
//! format :                                                      !
//!   char          indicate                                      !
//!    9       one numeric char                                   !
//!    8       one decimal char                                   ! 
//!    x       decimal sign (x = any char except 9 and 8          !
//! order : integer char, decimal sign, decimal char              !
//!                                                               !
//! samples :                                                     !
//! setformat("9999.88") give "1234.56"                           !
//! setformat("9,888") give "1,234"                               !
//! setformat("99") give "12"                                     !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Currency::setformat (const std::string sBP_SetFormat)
{
	int iNumInt;
	int iNumDec;
	std::string sDecimalSign;
	std::string sTemp;
	int iTemp;

	// reset error code
	iBPError = 0;

	// initialize values
	iNumInt = 0;
	iNumDec = 0;
	sDecimalSign = "";

	// copy format
	sTemp = sBP_SetFormat;

	// search for number of char for the integer part
	iTemp = sTemp.find_first_not_of ("9");
	if (iTemp >= 0)
		iNumInt = iTemp;
	else
		iNumInt = sTemp.length ();

	// assign decimal sign
	if (iTemp >= 0)
		sDecimalSign = sTemp [iTemp];

	// search for number of char for the decimal part
	sTemp = sTemp.substr (iTemp + 1);
	iTemp = sTemp.find_first_not_of ("8");
	if (iTemp >= 0)
		iNumDec = iTemp;
	else
		iNumDec = sTemp.length ();

	// store format
	BP_Currency::setformat (iNumInt, iNumDec, sDecimalSign,  "",  "",  "", "");

}



//-----------------------------------------------------------------
//!    BP_Currency::setstdformat : set standard format            !
//-----------------------------------------------------------------
//! iBP_SetNumInt : character to display before decimal sign      !
//! iBP_SetNumDec : character to display after decimal sign       !
//! sBP_SetFormat : name of the standard format                   !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!  standard ! decimal ! kilo ! currency ! leading ! null        !
//!   format  !  sign   ! sign !  symbol  !  char   ! fmt         !
//! --------------------------------------------------------      !
//!     US         .      none     none      none     none        !
//!     US$        .       ,        $        none     none        !
//!     FR         ,      none     none      none     none        !
//!     FR$        ,       .       euro      none     none        !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Currency::setstdformat (const int iBP_SetNumInt,
						   const int iBP_SetNumDec,
						   const std::string sBP_SetFormat)
{
	// reset error code
	iBPError = 0;
	
	if (sBP_SetFormat == "US")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ".",  "",  "",  "", "");
	else if (sBP_SetFormat == "US$")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ".",  ",",  "$",  "", "");
	else if (sBP_SetFormat == "FR")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ",",  "",  "",  "", "");
	else if (sBP_SetFormat == "FR$")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ",",  ".",  "euro",  "", "");
	else
		iBPError = 2;

}



//-----------------------------------------------------------------
//!    BP_Currency::getformat : get format of currency            !
//-----------------------------------------------------------------
//! return : BP_STRUCT_FORMAT structure                           !
//-----------------------------------------------------------------
BP_STRUCT_FORMAT BP_Currency::getformat (void)
{

	BP_STRUCT_FORMAT BPFormat;

	BPFormat.iBP_NumInt = BP_Fmt.iBP_NumInt;
	BPFormat.iBP_NumDec = BP_Fmt.iBP_NumDec;
	BPFormat.sBP_DecimalSign = BP_Fmt.sBP_DecimalSign;
	BPFormat.sBP_KiloSign = BP_Fmt.sBP_KiloSign;
	BPFormat.sBP_CurrencySymbol = BP_Fmt.sBP_CurrencySymbol;
	BPFormat.sBP_LeadChar = BP_Fmt.sBP_LeadChar;
	BPFormat.sBP_Null = BP_Fmt.sBP_Null;

	return BPFormat;

}


